import logging

from openlineage.airflow.extractors.base import TaskMetadata
from openlineage.airflow.extractors.snowflake_extractor import SnowflakeExtractor

from . import util

log = logging.getLogger(__name__)


class AlvinSFExtractor(SnowflakeExtractor):
    def __init__(self, operator):
        super().__init__(operator)
        util.print_alvin_environment(AlvinSFExtractor.__name__)

    # make all versions work
    def _get_hook(self):
        if hasattr(self.operator, "get_db_hook"):
            # Airflow v2
            return self.operator.get_db_hook()
        # Airflow v1
        return self.operator.get_hook()

    # change from
    # openlineage/airflow/extractors/snowflake_extractor.py
    # line 35
    def _get_database(self) -> str:
        print(f"PATCHING OPENLINEAGE FOR SNOWFLAKE: {str(self._get_hook())}")
        # return self.operator.get_db_hook()._get_conn_params()['database']
        return self._get_hook().database

    # change from
    # openlineage/airflow/extractors/snowflake_extractor.py
    # line 38
    def _get_authority(self) -> str:
        print(f"PATCHING OPENLINEAGE FOR SNOWFLAKE: {str(self._get_hook())}")
        # return self.operator.get_db_hook()._get_conn_params()['account']
        return self._get_hook().snowflake_conn_id

    # TODO figure out why `def extract` in the parent classes
    # does not allow to inject the `task_instance`
    # def extract(self) -> TaskMetadata:
    #     tm = super().extract()
    #     # TODO sort out Job Facet
    #     return tm

    def _get_query_ids(self):
        if hasattr(self.operator, "query_ids"):
            # Airflow v2
            return self.operator.query_ids
        # Airflow v1
        return self.operator.sfqids

    def extract_on_complete(self, task_instance) -> TaskMetadata:
        tm = super().extract_on_complete(task_instance)

        # TODO sort out Job Facet
        print("Alvin SNOWFLAKE TODO sort out Job Facet")
        db_hook = self._get_hook()
        conn_params = db_hook._get_conn_params()
        print(f"Alvin SNOWFLAKE ON COMPLETE - conn params: {conn_params}")
        query_ids = self._get_query_ids()
        print(f"Alvin SF query IDs on extract on complete? {query_ids}")
        tm.job_facets["alvin"] = {
            "snowflake_query_ids": query_ids,
        }
        return tm
