import os

import airflow
import openlineage

from . import alvin_vars

AIRFLOW_HOME = ""
OPENLINEAGE_URL = ""
OPENLINEAGE_API_KEY = ""
OPENLINEAGE_EXTRACTOR_BigQueryExecuteQueryOperator = ""
OPENLINEAGE_EXTRACTOR_SnowflakeOperator = ""

ENV_VARS_TO_MONITOR = []


def print_alvin_environment(prefix=""):
    if len(prefix) > 0:
        prefix = f"{prefix} "
    print(f"{prefix}=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=")
    for env_var in ENV_VARS_TO_MONITOR:
        # TODO comment out sensitive data e.g. the API KEY print!
        print(f"{prefix}Check this env var: {env_var}")
    print(f"{prefix}Package path for Airflow:\n{airflow.__path__}")
    print(f"{prefix}Package path for OpenLineage:\n{openlineage.__path__}")
    print(f"{prefix}=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=")


def set_global_vars():
    print_alvin_environment("Alvin Util")
    print("Setup the Alvin global variables (picking from the environment...")
    global AIRFLOW_HOME
    AIRFLOW_HOME = os.environ.get("AIRFLOW_HOME", "Not set! AIRFLOW_HOME")
    global OPENLINEAGE_URL
    OPENLINEAGE_URL = os.environ.get("OPENLINEAGE_URL", "Not set! OPENLINEAGE_URL")
    global OPENLINEAGE_API_KEY
    OPENLINEAGE_API_KEY = os.environ.get(
        "OPENLINEAGE_API_KEY", "Not set! OPENLINEAGE_API_KEY"
    )
    global OPENLINEAGE_EXTRACTOR_BigQueryExecuteQueryOperator
    OPENLINEAGE_EXTRACTOR_BigQueryExecuteQueryOperator = os.environ.get(
        "OPENLINEAGE_EXTRACTOR_BigQueryExecuteQueryOperator",
        "Not set! OPENLINEAGE_EXTRACTOR_BigQueryExecuteQueryOperator",
    )
    global OPENLINEAGE_EXTRACTOR_SnowflakeOperator
    OPENLINEAGE_EXTRACTOR_SnowflakeOperator = os.environ.get(
        "OPENLINEAGE_EXTRACTOR_SnowflakeOperator",
        "Not set! OPENLINEAGE_EXTRACTOR_SnowflakeOperator",
    )
    global ENV_VARS_TO_MONITOR
    ENV_VARS_TO_MONITOR = [
        AIRFLOW_HOME,
        OPENLINEAGE_URL,
        # TODO comment out this API KEY print!
        OPENLINEAGE_API_KEY,
        OPENLINEAGE_EXTRACTOR_BigQueryExecuteQueryOperator,
        OPENLINEAGE_EXTRACTOR_SnowflakeOperator,
    ]
    print("DONE setup the Alvin global variables (picking from the environment...")
    print_alvin_environment("Alvin Util")


def setup_environment_config():
    """
    Here, we just can follow the guides here to set the proper env variables!
    https://github.com/OpenLineage/OpenLineage/blob/main/integration/airflow/README.md
    """
    print("Alvin: setup the environment config (and env vars mapping)")
    # setup BigQuery OL extractor env vars
    for bq_op in alvin_vars.BQ_OPERATORS:
        curr_env_var_key = f"OPENLINEAGE_EXTRACTOR_{bq_op}"
        os.environ[
            curr_env_var_key
        ] = "alvin_lineage.alvin_bq_extractor.AlvinBQExtractor"
        print(f"Added this to the OL environment: {curr_env_var_key}")
    # setup Snowflake OL extractor env vars
    snowflake_env_var_key = "OPENLINEAGE_EXTRACTOR_SnowflakeOperator"
    os.environ[
        snowflake_env_var_key
    ] = "alvin_lineage.alvin_sf_extractor.AlvinSFExtractor"
    print(f"Added this to the OL environment: {snowflake_env_var_key}")
    # setup ALVIN_* vars from the env (via bash) => re-mapping to OPENLINEAGE_* via Python
    alvin_api_url = os.environ.get(alvin_vars.ALVIN__API_URL, "https://app.alvin.ai")
    os.environ[alvin_vars.OPENLINEAGE_URL] = alvin_api_url
    print(
        f"Mapped {alvin_vars.ALVIN__API_URL} to {alvin_vars.OPENLINEAGE_URL} as {alvin_api_url}"
    )
    alvin_api_key = os.environ.get(alvin_vars.ALVIN__API_KEY)
    if not alvin_api_key:
        print(f"The environment variable {alvin_vars.ALVIN__API_KEY} is not set!")
    else:
        os.environ[alvin_vars.OPENLINEAGE_API_KEY] = alvin_api_key
        print(f"Mapped {alvin_vars.ALVIN__API_KEY} to {alvin_vars.OPENLINEAGE_API_KEY}")
    set_global_vars()
    print("DONE Alvin: setup the environment config (and env vars mapping)")


# TODO perhaps Gorilla affects this global?
setup_environment_config()
